// Mapping: using something (input) to control something else (output)

void setup()
{
    size(400, 400); 
    
    /* From the referece we know this means we set the hue from 0 to 360,
    the saturation from 0 to 100, and the brightness from 0 to 100. */
    colorMode(HSB, 360, 100, 100);
    
    noStroke(); //this just says we don't want borders on our shapes by default
}

void draw()
{
   /*
   When Mapping we want our controller (input) value to be normalized.
   This means the range of its values should go from 0 to 1.
   Why??? Because, a range of 0 to 1 is easy to map to the desired 
   output range.
   
   So, let's use the mouse as our controller.
   We know mouseX is a special processing variable that will report the mouse's 
   pixel location along the X axis. Since our width is 400, mouseX will range
   from 0 to 399. Why not 1 to 400? Good question! This is because in programming,
   values are "zero-indexed" - this just means that everything starts at 0 instead
   of 1.
   
   So, if we want mouseX to range from 0 to 1 instead of 0 to 399, we just need to 
   divide its value by 399, right? Yes... but mouseX is an int, so we first need
   to convert it to a float so we can get a fractional result, i.e. 0.2, 0.7, etc.
  */
  
   // Ways to convert mouseX to a float:
   // 1) create a new float and set it equal to mouseX
   //float mX = mouseX; // now, we have a float that goes from 0 to 399
   //float mXn = mX/( width - 1 ); // here we have a "normalized" float that goes from 0.0 to 1.0
   
   // 2) Or use a "type-cast" and save a line
   float mXn = (float)mouseX/( width - 1 );
   
   // 3) Without converting mouseX to a float, you will have int/int = int, so only 0 and 1 are possible
   //float mXn = mouseX/(width-1);
   
   //Try each way of calculating mXn above and observe the results
   // 1) and 2) are equally valid - it's your preference for readability
   // 3) is incorrect since you can't represent fractions with ints
   println("normalized mouse X position is " + mXn);
   
   // Cool, now we have a normalized control value, we can map it to control anything!
   
   //How about the background hue, which ranges from 0 to 360?
   //Just multiply our normalized value by 360...
   background(mXn*360, 100, 100); //multiplying a float by an int gives a float
   
   //How about the size of a circle which ranges from 0 to width?
   ellipse(width/2, height/2, mXn*width, mXn*width);
   //Note: width/2 is an int divided by an int, but that's ok since the answer can be an int
   
   //Now, let's use another controller, the mouse's Y position, to control the brightness of the circle
   float mYn = (float)mouseY/(height-1); //normalized y position
   
   //fill(180, 100, mYn*100); //brightness maximum when mouse is at bottom of screen
   fill(180, 100, (1-mYn)*100); //brightness maximum when mouse is at top of screen
   
   /*That's another advantage to normalizing the conroll value: if we need to invert it
   we just subtract it from 1. */
   
   //Just draw the same sized ellipse
   ellipse(width/2, height/2, mXn*width, mXn*width); 
   
}